/*
 	Copyright (C) 2003 Frdric Giudicelli (contact_nos@yahoo.com).
	All rights reserved.

	This product includes cryptographic software written by Eric Young
	(eay@cryptsoft.com)

	This program is released under the GPL with the additional exemption that
	compiling, linking, and/or using OpenSSL is allowed.

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2 of the License.

	This program is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	this program; if not, write to the Free Software Foundation, Inc., 59 Temple
	Place, Suite 330, Boston, MA 02111-1307 USA
*/


#ifndef ENTITY_KEYSTORE_ASN1_H
#define ENTITY_KEYSTORE_ASN1_H

#include <mVector.h>
#include <ASN1/NewPKIObject.h>
#include <openssl/asn1.h>
#include "Entity.h"
#include <ASN1/Asn1Helper.h>

/*!
	This structure represent the local conf.
*/
typedef struct st_LOCAL_KEYSTORE_CONF
{
	ENTITY_CONF * conf;					//!< The conf.
	INTERNAL_PKI_CA * cas;				//!< The PKI internal CA.
}LOCAL_KEYSTORE_CONF;
DECLARE_ASN1_ITEM(LOCAL_KEYSTORE_CONF)

/*!
  This class is the representation of LOCAL_KEYSTORE_CONF
*/
class LocalKeystoreConf : public NewPKIObject, public LocalEntityConf
{
public:
	/*! \brief This is the constructor.
	 */
	LocalKeystoreConf();

	/*! \brief This is the constructor.
	 *  \param other [IN] the datas to copy from.
	 */
	LocalKeystoreConf(const LocalKeystoreConf & other);

	/*! \brief This is the destructor.
	 */
	~LocalKeystoreConf();


	/*! \brief This function converts a LOCAL_KEYSTORE_CONF to a LocalKeystoreConf.
	 *  \param Datas [IN] the datas to load from.
	 *  \return true on success, false on failure.
	 */
	bool load_Datas(const LOCAL_KEYSTORE_CONF * Datas);

	/*! \brief This function converts a LocalKeystoreConf to a LOCAL_KEYSTORE_CONF.
	 *  \param Datas [OUT] the datas to copy to.
	 *  \return true on success, false on failure.
	 */
	bool give_Datas(LOCAL_KEYSTORE_CONF ** Datas) const;

	/*! \brief This function clears all the internals.
	 */
	void Clear();

	/*! \brief This function returns the ASN1_ITEM of LOCAL_KEYSTORE_CONF.
	 *  \return The ASN1_ITEM.
	 */
	static const ASN1_ITEM * get_ASN1_ITEM();

	/*! \brief This function sets INTERNAL_PKI_CA::cas.
	 *  \param c_cas [IN] the datas.
	 *  \return true on success, false on failure.
	 */
	bool set_cas(const InternalPkiCa & c_cas);

	/*! \brief This function returns INTERNAL_PKI_CA::cas.
	 *  \return The value of INTERNAL_PKI_CA::cas.
	 */
	const InternalPkiCa & get_cas() const;

	/*! \brief This function returns INTERNAL_PKI_CA::cas.
	 *  \return The value of INTERNAL_PKI_CA::cas.
	 */
	InternalPkiCa & get_cas();


	/*! \brief This function sets ENTITY_CONF::conf.
	 *  \param c_conf [IN] the datas.
	 *  \return true on success, false on failure.
	 */
	bool set_conf(const EntityConf & c_conf);

	/*! \brief This function returns ENTITY_CONF::conf.
	 *  \return The value of ENTITY_CONF::conf.
	 */
	const EntityConf & get_conf() const;

	/*! \brief This function returns ENTITY_CONF::conf.
	 *  \return The value of ENTITY_CONF::conf.
	 */
	EntityConf & get_conf();


	/*! \brief This function converts the local instance to a Asn1EncryptSign.
	 *  \param cryptinfo [OUT] The result.
	 *  \param sig_pkey [IN] The signature private key.
	 *  \param crypt_pkey [IN] The encipherment public key.
	 *  \param sig_md [IN] The hash algorythm.
	 *  \param crypt_cypher [IN] The symetrical encipherment hash.
	 *  \return true on success, false on failure.
	 */
	bool to_SignEncrypt(Asn1EncryptSign & cryptinfo, const EVP_PKEY * sig_pkey, const EVP_PKEY * crypt_pkey, const EVP_MD * sig_md, const EVP_CIPHER * crypt_cypher) const;

	/*! \brief This function loads the local instance from a Asn1EncryptSign.
	 *  \param cryptinfo [IN] The signed and crypted datas.
	 *  \param sig_pkey [IN] The signature private key.
	 *  \param crypt_pkey [IN] The encipherment public key.
	 *  \return true on success, false on failure.
	 */
	bool from_SignEncrypt(const Asn1EncryptSign & cryptinfo, const EVP_PKEY * sig_pkey, const EVP_PKEY * crypt_pkey);


	/*! \brief This function copies a LocalKeystoreConf to another.
	 *  \param other [IN] the datas to copy from.
	 *  \return true on success, false on failure.
	 */
	bool operator=(const LocalKeystoreConf & other);
	/*! \brief This member is an empty instance of LocalKeystoreConf.
	 */
	static LocalKeystoreConf EmptyInstance;

private:
	InternalPkiCa m_cas;
	EntityConf m_conf;
	void resetAll();
	void freeAll();
protected:
};


#endif // #ifndef ENTITY_KEYSTORE_ASN1_H
